#####################################################

# <GLARED is an add-on for Blender that adds Light Leaks FX.>
# Copyright (C) <2023>  <Beniamino Della Torre e Alfonso Annarumma>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

# You can contact the creator of Glared here: blenderlensflare@gmail.com

#####################################################



import importlib
if "bpy" in locals():
    importlib.reload(function)
else:
    from . import (function)

import bpy
import os
from mathutils import Vector


from bpy.types import  Operator
from bpy_extras.io_utils import ExportHelper, ImportHelper

from bpy.props import EnumProperty, StringProperty, BoolProperty, IntProperty, CollectionProperty, FloatProperty, FloatVectorProperty, PointerProperty


def ShowMessageBox(message="", title="Message Box", icon='INFO', field='NONE'):
    def draw(self, context):
        idx = context.scene.lensflareitems_index

        layout = self.layout

        message = "Remove Preset?"

        row = layout.row()
        row.label(text=message)

        row = layout.row()

        row.operator("scene.glared_cancel", text="NO")
        row = layout.row()


        if field == 'PRESETCOMPOSITE':
            row.operator("glared.add_preset_composite", text="YES").remove_active = True

    bpy.context.window_manager.popup_menu(draw, title=title, icon=icon)


class SCENE_OT_Glared_popup_remove_preset(Operator):
    """Delete Current Settings Preset"""
    bl_idname = "scene.glared_popup_remove_preset"
    bl_label = "Remove Active Preset"

    field: StringProperty()

    def execute(self, context):
        # print ("ok")
        ShowMessageBox("Remove Preset?", "REMOVE PRESET", 'INFO', self.field)
        return {'FINISHED'}

class SCENE_OT_GlaredCancel(Operator):
    """Cancel Button"""
    bl_idname = "scene.glared_cancel"
    bl_label = "Cancel Button"

    def execute(self, context):
        return {'CANCELLED'}

class SCENE_OT_Glared_RealtimeFX(Operator):
    """Add a FX Nodegroup to the Compositor in the Last Position"""
    bl_idname = "scene.glared_realtime_fx"
    bl_label = "Real-time FX"

    auto_setup: BoolProperty(default=False, name="Auto Setup Compositing Nodes",
                         description="Automatically Create a Compositing Nodes Setup? PAY ATTENTION: This could create a bad node setup if you are already using compositor nodes")

    @classmethod
    def poll(cls, context):
        scene = context.scene
        if scene.node_tree:
            return "Glared" not in scene.node_tree.nodes
        else:
            return True

    def execute(self, context):
        scene = context.scene
        node_name = "Glared"
        if self.auto_setup:
            nodes = scene.node_tree.nodes
            links = scene.node_tree.links

            for node in nodes:
                nodes.remove(node)
            glared = nodes.new("CompositorNodeGroup")
            glared.name = node_name
            glared.node_tree = bpy.data.node_groups[node_name]
            composite = nodes.new("CompositorNodeComposite")

            footage = nodes.new("CompositorNodeRLayers")
            footage.location = Vector((-500, 0))
            glared.location = Vector((0, 0))
            composite.location = Vector((500, 0))

            links.new(footage.outputs[0], glared.inputs[0])
            links.new(glared.outputs[0], composite.inputs[0])


        #
        # message = "Check your composite node, you must have a connection to the output"
        # success = function.add_composite_node_group(context,node_name="Glared")
        # if not success:
        #     self.report({'ERROR'}, message)

        return {'FINISHED'}


    def invoke(self, context, event):
        if not "Glared" in bpy.data.node_groups:
            function.import_composite_node(node_name="Glared")
        success = function.add_composite_node_group(context,node_name="Glared")
        if not success:
            return context.window_manager.invoke_props_dialog(self)
        else:
            self.auto_setup = False
            return self.execute(context)
    def draw(self, context):


        row = self.layout
        col = row.column(align=True)
        col.label(text="Check your compositing nodes,")
        col.label(text="you must have a connection to the output")

        row = self.layout
        row.prop(self, "auto_setup")
        if self.auto_setup:
            row = self.layout
            row.label(text="This operation will delete your node setup")

class SCENE_OT_Glared_RemoveRealtimeFX(Operator):
    """Remove the FX Nodegroup"""
    bl_idname = "scene.glared_remove_realtime_fx"
    bl_label = "Real-time FX"


    @classmethod
    def poll(cls, context):
        scene = context.scene
        if scene.node_tree:
            return "Glared" in scene.node_tree.nodes
        else:
            return False

    def execute(self, context):
        function.remove_composite_node_group(context, node_name="Glared")


        return {'FINISHED'}




class SCENE_OT_Glared_ExportPreset(Operator, ExportHelper):
    """Export Preset to zip file"""
    bl_idname = "export.glared_preset_data"  # important since its how bpy.ops.import_test.some_data is constructed
    bl_label = "Export Glared Preset"

    # ExportHelper mixin class uses this
    filename_ext = ".zip"

    filter_glob: StringProperty(
        default="*.zip",
        options={'HIDDEN'},
        maxlen=255,  # Max internal buffer length, longer would be clamped.
    )
    type: StringProperty(
        default="glared",
        options={'HIDDEN'},
        maxlen=255,  # Max internal buffer length, longer would be clamped.
    )


    @classmethod
    def poll(cls, context):
        preset_dir = bpy.utils.user_resource('SCRIPTS', path="presets\\" + "glared")
        return os.path.exists(preset_dir)


    def execute(self, context):
        preset_dir = bpy.utils.user_resource('SCRIPTS', path="presets\\" + self.type)

        return function.zip_presets(preset_dir, self.filepath)


class SCENE_OT_Glared_ImportPreset(Operator, ImportHelper):
    """Export Preset to zip file"""
    bl_idname = "import.glared_preset_data"  # important since its how bpy.ops.import_test.some_data is constructed
    bl_label = "Import Glared Preset"

    # ImportHelper mixin class uses this
    filename_ext = ".zip"

    filter_glob: StringProperty(
        default="*.zip",
        options={'HIDDEN'},
        maxlen=255,  # Max internal buffer length, longer would be clamped.
    )
    type: StringProperty(
        default="glared",
        options={'HIDDEN'},
        maxlen=255,  # Max internal buffer length, longer would be clamped.
    )

    # List of operator properties, the attributes will be assigned
    # to the class instance from the operator settings before calling.

    def execute(self, context):
        preset_dir = bpy.utils.user_resource('SCRIPTS', path="presets\\" + self.type)
        if not os.path.exists(preset_dir):
            os.makedirs(preset_dir)
        return function.unzip_presets(preset_dir, self.filepath)


class SCENE_OT_GlaredResetWarning(Operator):
    """Reset All Warning message"""
    bl_idname = "scene.glared_reset_warning"
    bl_label = "Reset all warning message"

    def execute(self, context):
        preferences = context.preferences.addons[__package__].preferences
        preferences.glare_alert = False
        preferences.multy_scene_warning = False
        return {'FINISHED'}
